//-----------------------------------------------------------------
//  Demo program to test the SRAM disk direct I/O
//  Advantech eAutomation Division
//  01-27-2010
//
//  Version: 1.0.0
//  Author:  Jianfeng.Dai
//  ChangeLog:
//  	--version 1.0.0  initial version.
//-----------------------------------------------------------------  

#include   <sys/types.h>
#include   <sys/stat.h>
#include   <unistd.h>
#include   <errno.h>
#include   <fcntl.h>
#include   <string.h>
#include   <stdio.h>

#define BLOCK_SIZE  1024
#define DATA_SIZE    BLOCK_SIZE

typedef char _aligned_char __attribute__ ((aligned (BLOCK_SIZE)));

char odata[DATA_SIZE] __attribute__ ((aligned (BLOCK_SIZE)));   // original data
char wdata[DATA_SIZE] __attribute__ ((aligned (BLOCK_SIZE)));
char rdata[DATA_SIZE] __attribute__ ((aligned (BLOCK_SIZE)));


void usage( void )
{
	printf("testraw usage:\n");
	printf("	./testraw size\n");
	printf("	For Example: UNO-3074 has 512K sram, to test it as follows:\n");
	printf("	./testraw 512\n");
}

int main ( int argc, char **argv )
{
   int i, cursor, count, sramsize,sramfd, retval=0;
   unsigned char command[50];

   FILE *psramfd;
   off_t ret;
   
   if ( argc <2 )
   {
   		usage();
   		return 0;
   }
   sramsize = atoi( argv[ 1 ]);
   if ((sramfd = open( "/dev/fs0", O_RDWR  )) == -1 )
   {
      printf("Error to open %s \n", argv[ 1 ]);
      retval = 2;
      goto out;
   } 

   /* prepare test pattern */
   for(i=0; i<DATA_SIZE; i++)
       wdata[i] = (i%2) ? 0x55 : 0xAA;

   /*Raw Access to sram */
   for ( i = 0; i < sramsize; i++ )
   {   
     printf("\rTesting direct I/O  in NO. %dk of sram", i+1);
      cursor = i * DATA_SIZE;
      if ( (ret = lseek( sramfd, cursor, SEEK_SET )) < 0 )
      {
         printf("\nError to perform lseek: ret = %d", ret);
         retval = 5;
         goto err;
      }
   
      // backup sram device data
      count = read( sramfd, odata, sizeof(odata));
      if ( count != sizeof(odata))
      {
         printf("\nError to backup data, %d\n", count);
         retval = 6;
         goto err;
      }
      
      /* write test pattern to sram */
      lseek( sramfd, cursor, SEEK_SET );
      count = write( sramfd, wdata, sizeof(wdata) );
      if ( count != sizeof( wdata ))
      {
         printf("\nError to write test pattern (%d, %d)\n",
                strlen(wdata), count);
         retval = 7;
         goto err;  
      }
 
      /* read back test pattern from sram */     
      lseek( sramfd, cursor, SEEK_SET );
      count = read( sramfd, rdata, sizeof(rdata));
      if ( count != sizeof(rdata))
      {
         retval = 8;
         printf("\nError to read test pattern, %d\n", count);
         goto err;
      }

      /* compare test pattern */
      if ( !memcmp( rdata, wdata, count ) )
      {
         lseek( sramfd, cursor, SEEK_SET );
         /* restore data to sram */
         count = write( sramfd, odata, sizeof(odata) );
         if ( count != sizeof( odata ))
         {
            printf("\nError to restore data (%d, %d)\n",
                   sizeof(odata), count);
            retval = 9;
            goto err;  
         }
         continue;
      }
      else
      {  
         printf("\nError!\n");
         retval = 10;
         goto err;
      }
   }
   printf("\nThe device is tested ok, SRAM is ready for use!\n");

err:
   close( sramfd );
out:
   return retval;
}
   
